function [bta] = NetCox(Data, lambda, alpha, d, S)

    % Data: a m-by-n matrix, m is the number of genes, n is the number of
    % samples. Note: the sample dimesion of Data is sorted by the survival time (Times)
    % d: censor information, 1 is uncensored, 0 is censored.
    % S: normalized network information

    d = double(d);
    % m: number of genes
    % n: number of samples
    [m,n] = size(Data);

    %maximum interation for the outer loop
    maxIter1 = 200;

    %maximum iteration for the inner loop (most of time it less than 10 iterations)
    maxIter2 = 200;

    %gamma = zeros(length(lamda),length(alpha),n);
    bta = zeros(length(lambda),length(alpha),m);

    inner_conv_val = 1e-8;
    outer_conv_val = 1e-6;

    par_struct = struct;
    idx = 1;
    % for every alpha
    for nn = 1:length(alpha)

       for mm = 1:length(lambda)
            par_struct(idx).mm = mm;
            par_struct(idx).nn = nn;
            par_struct(idx).alpha_v = alpha(nn);
            par_struct(idx).lambda_v = lambda(mm);
            par_struct(idx).bta = zeros(m,1);
            idx = idx + 1;
        end
    end
    clear idx
    %
    parfor kk = 1:length(par_struct) % for every lambda
        warning 'off';
        alpha_v  = par_struct(kk).alpha_v;
        lambda_v = par_struct(kk).lambda_v;
        %
        % laplacian matrix
        L = (1-alpha_v)*(eye(m) - S) + alpha_v*eye(m);
        % av: data transformed to laplacian space ??
        Z = Data'/L*Data;
        %
        fprintf('      going to calculate for alpha=%.3E and lambda=%.3E\n',alpha_v,lambda_v);
        gamma_outer = rand(n,1);         % random start ??
        lp          = zeros(maxIter1,1); % log-partial likelihood
        pena        = zeros(maxIter1,1); % penalization

        C = Z * gamma_outer;

        M = zeros(n,n);
        for j = 1:n % for every n number of samples
            M(j,:) = C' - C(j);
        end
        M = exp(M);

        tmp = zeros(n,1);
        for k = 1:n
            IX = find(d(1:k)==1);
            for p = 1:length(IX)
                tmp(k) = tmp(k) + 1/sum(M(k,IX(p):n));
            end
        end

        lnh = zeros(n,1);
        for p = 1:n-1
            lnh(p) = -C(p) - log(1 + sum(M(p,p+1:n)));
        end
        lnh(n) = -C(n);

        lp_old = 0;

        for i = 1:maxIter1
            gamma_outer_old = gamma_outer;

            % penalization
            pena(i) = (1/2)*lambda_v*(gamma_outer'*Z*gamma_outer);
            % log-partial likelihood
            lp(i) = sum(-tmp + d.*(lnh + C)) - pena(i);

            gamma_inner = gamma_outer;
            lp_inner_old = 0;

            % Newton-Raphson
            for t = 1:maxIter2
                gamma_inner_old = gamma_inner;
                dta = d -tmp;
                if any(isnan(dta))
                    continue
                end
                % first derivative
                firstD = Z*dta - lambda_v*Z*gamma_inner;
                D = diag(tmp); % diagonal matrix
                % second derivative
                secondD = -Z*D*Z'- lambda_v*Z;
                gamma_inner = gamma_inner - secondD \ firstD;
                tmp = tmp.*exp(Z*gamma_inner-Z*gamma_inner_old);

                pena_inner = (1/2)*lambda_v*(gamma_inner'*Z*gamma_inner);
                lp_inner = sum(-tmp + d.*(lnh + C)) - pena_inner;

                % convergence check
                if (max(abs(gamma_inner-gamma_inner_old)) < inner_conv_val  || abs(lp_inner-lp_inner_old) < inner_conv_val )
                    break;
                end

                lp_inner_old = lp_inner;

                if t == 1000
                    fprintf('        NetCox: The Newton-Raphson is not converge, alpha=%.3E | lambda=%.3E\n',alpha_v,lambda_v)
                end

            end % Newton-Raphson
            gamma_outer = gamma_inner;

            % if converge then break
            if (max(abs(gamma_outer-gamma_outer_old)) < outer_conv_val || abs(lp(i)-lp_old) < outer_conv_val )
                break;
            end
            %update the baseline hazard
            lp_old = lp(i);

            C = Z * gamma_outer;

            M = zeros(n,n);
            for j = 1:n
                M(j,:) = C' - C(j);
            end
            M = exp(M);

            tmp = zeros(n,1);
            for k = 1:n
                IX = find(d(1:k)==1);
                for p = 1:length(IX)
                    tmp(k) = tmp(k) + 1/sum(M(k,IX(p):n));
                end
            end
            lnh = zeros(n,1);
            for p = 1:n-1
                lnh(p) = -C(p) - log(1 + sum(M(p,p+1:n)));
            end
            lnh(n) = -C(n);
        end
        par_struct(kk).bta(:) = L\Data*gamma_outer;
        %gamma(mm,nn,:) = gamma_outer;
        fprintf('      finished in %d iterations (alpha: %.3E, lambda: %.3E, i: %d, t: %d) with loglike: %f | min bta = %f | max bta = %f \n',...
            i*maxIter1-maxIter1 + t,alpha_v,lambda_v,i,t,loglike(Data', par_struct(kk).bta, d), min(par_struct(kk).bta), max(par_struct(kk).bta));
        
    end
    clear kk
    
    for kk = 1:length(par_struct)
        mm = par_struct(kk).mm;
        nn = par_struct(kk).nn;
        bta(mm,nn,:) = par_struct(kk).bta; 
    end

