
rm(list = ls())

# taken from running elastic-net manually to all 4 datasets and combining lambda values
lambda_raw <- c(seq(.5,9.5,1))
lambda_raw <- as.vector(sapply(10^(-6:6), function(e) {e * lambda_raw}))
lambda_raw <- c(lambda_raw, 
                0.273324552, 0.249043152, 0.206760000, 0.197362439, 
                0.179829304, 0.163853764, 0.136034273, 0.129851306, 
                0.118315674, 0.107804835, 0.089501468, 0.085433488, 
                0.077843813, 0.070928383, 0.058885990, 0.056209530, 
                0.051216031, 0.046666140, 0.038743049, 0.036982118, 
                0.033696728, 0.030703204, 0.025490339, 0.024331764, 
                0.022170197, 0.020200658, 0.016770941, 0.016008676, 
                0.014586509, 0.013290684, 0.011034159, 0.010532640, 
                0.009596949, 0.008744383, 0.007259740, 0.006929774, 
                0.006314152, 0.005753220, 0.004776424, 0.004559328, 
                0.004154290, 0.003785234, 0.003142568, 0.002999734)
params.lambda <- unique(sort(lambda_raw))
#
require('glmnet')
require('R.matlab')
require('survival')
require('futile.logger')

flog.appender(appender.tee('logger.txt'))

# Parallel
library(doMC) # parallel computing
library(foreach)
registerDoMC(cores=4) 

base_dir      <- 'data/train'
base_dir.raw  <- 'data/generated_source'
#
data_type_vec <- c('common_bono',            'common_tcga',
                   'common_funlinkage_bono', 'common_funlinkage_tcga', 
                   'common_tothill',         'common_funlinkage_tothill')
#data_type_vec <- c('common_funlinkage_bono')
#data_type_vec <- c('common_bono','common_tcga','common_funlinkage_tcga')
k_vec         <- c('01')
for ( k in k_vec ) {
  foreach(data_type = data_type_vec) %dopar% {
  # for ( data_type in  ){
    data_prefix <- paste0(data_type,'_',k,'k')
    
    infiles  <- dir(base_dir, pattern=paste0(data_prefix,'_[0-9][0-9]g[.]mat$'))
    
    #params.lambda <- c(1e-5,1e-4)
    params.nlambda <- 10
    if (k == '10') {
      params.nlambda <- params.nlambda / 2
    }
    results        <- NULL
    #
    params.alpha   <- c(0,1)
    params.nalpha  <- length(params.alpha)
    params.nlambda <- length(params.lambda)
    
    lambda_vec <- array(0,c(length(infiles), params.nlambda))
    alpha_vec  <- array(0,c(length(infiles), params.nalpha))
    
    flog.info(paste0('processing: \'', data_prefix, '*\' with \'',length(infiles),'\' files.'))
    for(filename in infiles) {
      flog.info(paste0('  loading \'', filename, '\''))
      # read data in matlab format that has idx for training set
      dat       <- readMat( paste0(base_dir,"/",filename) )
      # read actual data
      dat.raw   <- readMat( paste0(base_dir.raw, '/', gsub('(common(_funlinkage)?_[a-zA-Z]+)_[0-9]+k_[0-9]+g[.]mat', '\\1.mat', filename) ))
      # convert idx.train to logical to be used as index
      idx.train <- as.logical(dat$idx.train)
      # get data
      xdata     <- t(dat.raw$mydata[,,1]$Data.N)[idx.train,]
      # get the time and status variables
      ydata_aux <- cbind( time = dat.raw$mydata[,,1]$Times[idx.train], status = dat.raw$mydata[,,1]$d[idx.train])
      ydata <- ydata_aux
      xdata <- xdata[ydata_aux[,1] > 0,]
      ydata <- ydata[ydata_aux[,1] > 0,]
      # number of cv set
      i     <- dat$i
      # S_aux     <- dat$S
      # remove big variable, as all data has been read
      rm(dat) 
      rm(dat.raw)
      # create results structure, as it was not possible to determine before the first loop
      if (is.null(results)) {
        results <- array(0,c(length(infiles),params.nlambda, length(params.alpha), dim(xdata)[2]))
      }
      flog.info('  calculating alpha and lambda...')
      #
      # for each alpha and lambda, determine the glmnet
      for (mm in 1:length(params.alpha)) {
        # set the alpha value
        alpha_v = params.alpha[mm]
        # get local results
        #temp_results = cv.glmnet( xdata, ydata, family='cox', alpha=alpha_v, nlambda=params.nlambda,
        #                          standardize=TRUE, parallel = TRUE)$glmnet.fit
        # temp_results = glmnet(xdata, ydata, family='cox', alpha = alpha_v, nlambda = 100, standardize = T)
        #
        temp_results <- glmnet(xdata, ydata, family='cox', alpha = alpha_v, lambda = params.lambda, standardize = T)
        # save results
        for (nn in 1:length(temp_results$lambda)) {
          results[i,nn,mm,] <- temp_results$beta[,nn]
        }
      }
      flog.info('  storing in variable...')
      lambda_vec[i, 1:length(temp_results$lambda)] <- temp_results$lambda
      alpha_vec[i,]  <- params.alpha
    }
    flog.info('writing results to disk...')
    # save results to matlab format
    writeMat(paste0('data/results/r_results_',data_prefix,'.mat'),r_results=results,lambda_vec=lambda_vec,alpha_vec=alpha_vec)
  }
}
